#!/usr/bin/env python
# encoding: utf-8
"""
befriender.py

This object holds the socket descriptor for the 
befriender, and associated information. We can send
messages to remote befrienders with this object

Created by Elliot Pennington on 2008-11-16.
"""

import threading

class Befriender:
    
    """
    This class represents a single befriender login to the system.
    
    Attributes:
        login: the login id of the befriender.
        __sockfd: the socket descriptor for the connection to this befriender's 
                    machine.
        available: A boolean to describe whether or not the befriender is available
                    for conversation.
    """
    
    def __init__(self, login, sockfd):
        
        """ 
        Accept the login id and the socket descriptor. 
        Set the befriender as available
        """
        
        self.login = login
        self.__sockfd = sockfd
        self.available = True
        
    def getSock(self):
        """ Return the socket descriptor for this befriender """
        return self.__sockfd
    
class Befrienders:
    
    """
    This class represents the set of all befrienders logged into the system.
    
    It contains a list of Befriender objects. This class will iterate through 
        it's own list of befrienders, and remember its iteration state. In 
        addition, if you call next(), when at the end of the list, it will wrap
        back to the beginning. This is used when determining which befriender
        to route a new client message to. This class is threadsafe. The internal 
        list is locked upon access.

    Attributes:
        array_lock: This a semaphore for locking the internal list on access. 
        __index: the iteration state of the array.
        __array: the internal array to store the Befriender objects.
    """
    
    def setUp(self):
        
        """ initialize the lock, the index, and the array."""
        
        self.array_lock = threading.RLock()
        self.__index = 0
        self.__array = []
        
    def __init__(self):

        self.setUp()

    def empty(self):
        """
        Tell us if there are any logged in Befrienders.
        
        Returns true if there are no logged in Befrienders.
        False otherwise.
        """
        self.array_lock.acquire()
        if len(self.__array) == 0:
            self.array_lock.release()
            return True
        self.array_lock.release()
        return False

    def add(self, befriender):

        """
        Add a befriender to the befriender list. This is called whenever a new
            befriender logs into the system. When adding, the list is locked
            against multiple access for thread safety.
        
        Args:
            befriender: the Befriender object to add to the list. 
        """
        
        self.array_lock.acquire()
        try:
            self.__array.append(befriender)
        finally:
            self.array_lock.release()
        
    def remove(self, sock):
        
        """
        Remove a befriender from the list. This is called whenever a befriender
            logs out of the system.
            
        Args:
            sock: the socket associated with the befriender to remove.
        """
        
        self.array_lock.acquire()
        try:
            for befriender in self.__array:
                if (befriender.getSock() == sock):
                    self.__array.remove(befriender)
        finally:
            self.array_lock.release()
    
    def next(self):
        
        """
        Iterate through the Befrienders list by one position. If it reaches the end of
            the list, it wraps back to the beginning. Note that the iteration state is 
            stored in the class, so it always increments from the last befriender found.
        """
        
        self.__index += 1
        if (self.__index >= len(self.__array)):
            self.__index = 0
        self.array_lock.acquire()
        try:
            #TODO(esp): set this back to index instead of taking the last logged in befriender
            return self.__array[len(self.__array) - 1]
        finally:
            self.array_lock.release()

if __name__ == '__main__':
    unittest.main()